/* "eqcs_gate_rep.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef __GNUG__
#pragma implementation
#endif

#include <assert.h>
#include "eqcs_gate_rep.h"

/*****************************************************************************\
*	gate representation														  *
\*****************************************************************************/

EqcsGateRep::EqcsGateRep(int nqubits) : _args(nqubits)
	{ _maxbit = 0; }

EqcsGateRep::~EqcsGateRep()
	{}

void EqcsGateRep::set(int i, int bit)
{
	assert(0 <= i && i < _args.size());

	// Test whether this is the leftmost bit.
	if (_maxbit < bit)
		_maxbit = bit;

	// Add the current bit to the argument list.
	_args[i] = bit;
	_bits |= bitset_t::bits2bitset(1ul << bit);
}

int EqcsGateRep::bitset2index(const bitset_t &bitset) const
{
	int index = 0;					// The result.

	for (int i = 0; i < _args.size(); ++i)
		if (bitset.test(_args[i]))
			index |= 1 << i;

	assert(0 <= index && index < dim());

	return index;
}

EqcsGateRep::bitset_t EqcsGateRep::index2bitset(int index) const
{
	bitset_t bitset(_maxbit + 1);	// The result.

	for (int i = 0; i < _args.size(); ++i)
		if (index & (1 << i))
			bitset.set(_args[i]);

	return bitset;
}
