/* "eqcs_lambda_rep.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifdef __GNUG__
#pragma implementation
#endif

#include <assert.h>
#include <iostream.h>
#include "eqcs_sep.h"
#include "eqcs_state.h"
#include "eqcs_state_rep.h"
#include "eqcs_lambda_rep.h"

/*****************************************************************************\
*	lambda representation (quant-ph/9503016)								  *
\*****************************************************************************/

EqcsLambdaRep::EqcsLambdaRep(const complex_t &u00, const complex_t &u01,
	const complex_t &u10, const complex_t &u11, int m) :
	_u00(u00), _u01(u01), _u10(u10), _u11(u11), base_t(m + 1)
	{}

EqcsLambdaRep::~EqcsLambdaRep()
	{}

EqcsStateRep *EqcsLambdaRep::apply(const EqcsStateRep *state)
{
	typedef EqcsStateRep::terms_t terms_t;

	const terms_t &terms = state->_terms;

	// Compute the result state's number of qubits.
	int n = maxbit() + 1;

	if (n < state->nqubits())
		n = state->nqubits();

	// Create an empty result state.
	EqcsStateRep *ret = new EqcsStateRep(n, 0);

	for (terms_t::const_iterator p = terms.begin(); p != terms.end(); ++p) {
		int row = bitset2index(p->first);

		if (row == dim() - 2) {
			bitset_t i = (p->first & ~bits()) | index2bitset(row);
			bitset_t j = (p->first & ~bits()) | index2bitset(row + 1);

			assert(i == p->first);

			// Find the corresponding second term.
			terms_t::const_iterator q = terms.find(j);
/*
if (q != terms.end()) {
	cout << "c'[" << i << "] = " <<
		_u00 << "c[" << i << "] + " << _u01 << "c[" << j << "]" << endl;
	cout << "c'[" << i << "] = " <<
		_u10 << "c[" << i << "] + " << _u11 << "c[" << j << "]" << endl;
}
else {
	cout << "c'[" << i << "] = " <<
		_u00 << "c[" << i << "]" << endl;
	cout << "c'[" << i << "] = " <<
		_u10 << "c[" << i << "]" << endl;
}
*/
			// Calculate the first row.
			complex_t c = _u00 * p->second;

			if (q != terms.end())
				c += _u01 * q->second;

			if (0.0 != c)
				ret->_terms[i] = c;

			// Calculate the second row.
			c = _u10 * p->second;

			if (q != terms.end())
				c += _u11 * q->second;

			if (0.0 != c)
				ret->_terms[j] = c;
		}
		else if (row == dim() - 1) {
			bitset_t i = (p->first & ~bits()) | index2bitset(row - 1);
			bitset_t j = (p->first & ~bits()) | index2bitset(row);

			assert(j == p->first);

			// Find the corresponding first term.
			terms_t::const_iterator q = terms.find(i);
/*
if (q != terms.end()) {
	cout << "c'[" << i << "] = " <<
		_u00 << "c[" << i << "] + " << _u01 << "c[" << j << "]" << endl;
	cout << "c'[" << i << "] = " <<
		_u10 << "c[" << i << "] + " << _u11 << "c[" << j << "]" << endl;
}
else {
	cout << "c'[" << i << "] = " <<
		_u00 << "c[" << j << "]" << endl;
	cout << "c'[" << i << "] = " <<
		_u10 << "c[" << j << "]" << endl;
}
*/
			// Calculate the first row.
			complex_t c = _u01 * p->second;

			if (q != terms.end())
				c += _u00 * q->second;

			if (0.0 != c)
				ret->_terms[i] = c;

			// Calculate the second row.
			c = _u11 * p->second;

			if (q != terms.end())
				c += _u10 * q->second;

			if (0.0 != c)
				ret->_terms[j] = c;
		}
		else
			ret->_terms[p->first] = p->second;
	}

	return ret;
}

ostream &EqcsLambdaRep::put(ostream &os) const
{
	const complex_t zero = 0.0;
	const complex_t one = 1.0;
	const unsigned long d = dim();
	EqcsSep newline("\n");
	EqcsSep tab("\t");

	// The trivial rows.
	for (unsigned long row = 0; row < (d - 2); ++row) {
		os << newline;
		tab.reset();

		for (unsigned long col = 0; col < d; ++col)
			os << tab << (row == col ? one : zero);
	}

	// The first non-trivial row.
	os << newline;
	tab.reset();

	for (unsigned long col = 0; col < (d - 2); ++col)
		os << tab << zero;

	os << tab << _u00 << tab << _u01;

	// The second non-trivial row.
	os << newline;
	tab.reset();

	for (unsigned long col = 0; col < (d - 2); ++col)
		os << tab << zero;

	os << tab << _u10 << tab << _u11;

	return os;
}
