/* "eqcs_state.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_STATE_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_STATE_H_

#include <complex>
#include "eqcs_bitset.h"

class ostream;
class EqcsStateRep;

/*****************************************************************************\
*	state																	  *
\*****************************************************************************/

class EqcsState: public EqcsHandle {
public:
	typedef EqcsHandle base_t;
	typedef EqcsStateRep rep_t;
	typedef EqcsBitset bitset_t;
	typedef complex<double> complex_t;
	typedef EqcsOManip2<EqcsState, int, int> omanip2_t;

	EqcsState(rep_t *rep=0);
	EqcsState(unsigned long bits);

	// Define a partial order.
	bool operator<(const EqcsState &state) const;

	// Negate the state.
	EqcsState operator-() const;

	// Add a state.
	EqcsState operator+=(const EqcsState &state) const;
	EqcsState operator-=(const EqcsState &state) const;
	// Multiply by a complex number.
	EqcsState operator*=(const complex_t &c) const;

	// Measure some bits.
	EqcsState collapse(unsigned long bits) const;

	ostream &put_bits(ostream &os, int nregs, int regwidth) const;
	ostream &put_nums(ostream &os, int nregs, int regwidth) const;

	// Some technical uitilities.
	inline rep_t *operator->() const
		{ return (rep_t *)EqcsHandle::rep(); }
	inline rep_t *rep() const
		{ return (rep_t *)EqcsHandle::rep(); }
};

// Add two states.
EqcsState operator+(const EqcsState &l, const EqcsState &r);
EqcsState operator-(const EqcsState &l, const EqcsState &r);
// Multiply a state by a complex number.
EqcsState operator*(const EqcsState::complex_t &l, const EqcsState &r);
EqcsState operator*(const EqcsState &l, const EqcsState::complex_t &r);

ostream &operator<<(ostream &os, const EqcsState &state);
EqcsState::omanip2_t bits(const EqcsState &state, int nregs, int regwidth);
EqcsState::omanip2_t nums(const EqcsState &state, int nregs, int regwidth);

#endif	//	_EQCS_STATE_H_
