/* "hello.cc"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include <iostream.h>
#include "eqcs_qc.h"
#include "eqcs_lambda.h"

int main()
{
	EqcsGateArray program;		// A gate array.

	// Create a controlled not gate (a special lamda(1) gate).
	EqcsLambda cnot(0.0, 1.0, 1.0, 0.0, 1);

	cnot.set(0, 1);				// The working bit is 1.
	cnot.set(1, 5);				// The controlling bit is 5.

	// Add the controlled not gate to the gate array.
	program.push_back(cnot);

	// Create the state |0010>|0010>.
	EqcsState state = (1ul << 5) | (1ul << 1);
	cout << "the input:  " << bits(state, 2, 4) << endl;

	// Create a quantum computer initially in state |0010>|0010>.
	EqcsQc qc(state);

	// Run the program.
	qc.perform(program);

	// Print the result.
	cout << "the result: " << bits(qc.state(), 2, 4) << endl;
}
